/*
Szymon Rusinkiewicz
Princeton University

xshade-make.cc
Create .xsh files, which contain a mesh plus several levels of
pre-smoothed normals, stored in an efficient format.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xsh.h"
#include "TriMesh_algo.h"


// Compute smoothed normals, etc.
void compute_xshdata(TriMesh *mesh, XSHdata *data)
{
	const float freqmult = M_SQRT2;
	const float smooth_multiplier = sqrt(sqr(freqmult) - 1.0f);

	float sigma = 0.4f * mesh->feature_size();
	float coarsest = 0.05f * mesh->bsphere.r;
	int nsmooth = (int) round(log(coarsest / sigma) / log(freqmult));

	data->sigmas.resize(nsmooth + 1);
	data->snormals.resize(nsmooth + 1);
	data->spdir.resize(nsmooth + 1);
	data->smeancurv.resize(nsmooth + 1);

	int nv = mesh->vertices.size();
	for (int pass = 0; pass <= nsmooth; pass++) {
		fprintf(stderr, "\nPass %d of %d...\n\n", pass, nsmooth);
		if (pass == 0) {
			mesh->need_normals();
		} else {
			diffuse_normals(mesh, sigma * smooth_multiplier);
			sigma *= freqmult;
		}
		mesh->pdir1.clear();  mesh->pdir2.clear();
		mesh->curv1.clear();  mesh->curv2.clear();
		mesh->need_curvatures();

		data->sigmas[pass] = sigma;
		data->snormals[pass] = mesh->normals;
		data->spdir[pass].reserve(nv);
		data->smeancurv[pass].reserve(nv);
		float pscale = 10.0f * sqr(sigma);
		for (int i = 0; i < nv; i++) {
			float &c1 = mesh->curv1[i];
			float &c2 = mesh->curv2[i];
			float s = min(pscale * (c1*c1 - c2*c2), 1.0f);
			data->spdir[pass].push_back(s * mesh->pdir1[i]);
			float H = 0.5f * sigma * (c1 + c2);
			data->smeancurv[pass].push_back(H);
		}
	}
}


void usage(const char *myname)
{
	fprintf(stderr, "Usage: %s in.ply\n", myname);
	exit(1);
}


int main(int argc, char *argv[])
{
	if (argc < 2)
		usage(argv[0]);

	char *filename = argv[1];
	TriMesh *mesh = TriMesh::read(filename);
	if (!mesh)
		usage(argv[0]);

	char *xshfilename = new char[strlen(filename) + 5];
	strcpy(xshfilename, filename);
	char *dot = strrchr(xshfilename, '.');
	if (!dot)
		dot = strrchr(xshfilename, '\0');
	strcpy(dot, ".xsh");

	mesh->need_tstrips();
	reorder_verts(mesh);
	mesh->need_bsphere();

	XSHdata *data = new XSHdata;
	compute_xshdata(mesh, data);
	xsh_write(xshfilename, mesh, data);
}

